@@ -15,6 +15,7 @@
 #include "decode.h"
 #include "err.h"
 #include "hosts_db.h"
+#include "linktypes.h"
 #include "localip.h"
 #include "now.h"
 #include "opt.h"
@@ -152,14 +153,12 @@ static void cap_start_one(struct cap_iface *iface, const int promisc) {
 
    /* Work out the linktype and what snaplen we need. */
    linktype = pcap_datalink(iface->pcap);
-   verbosef("linktype is %d", linktype);
+   verbosef("linktype is %d (%s)", linktype, get_linktype_name(linktype));
    if ((linktype == DLT_EN10MB) && opt_want_macs)
       hosts_db_show_macs = 1;
    iface->linkhdr = getlinkhdr(linktype);
    if (iface->linkhdr == NULL)
       errx(1, "unknown linktype %d", linktype);
-   if (iface->linkhdr->decoder == NULL)
-      errx(1, "no decoder for linktype %d", linktype);
    snaplen = getsnaplen(iface->linkhdr);
    if (opt_want_pppoe) {
       snaplen += PPPOE_HDR_LEN;
@@ -522,8 +521,6 @@ void cap_from_file(const char *capfile) {
    iface.linkhdr = getlinkhdr(linktype);
    if (iface.linkhdr == NULL)
       errx(1, "unknown linktype %d", linktype);
-   if (iface.linkhdr->decoder == NULL)
-      errx(1, "no decoder for linktype %d", linktype);
 
    cap_set_filter(iface.pcap, iface.filter);
 
